@@ -1,21 +1,28 @@
 import os
 from functools import wraps
-from itertools import chain
 from threading import Lock
+from xml.sax.saxutils import escape
 
 import numpy as np
 
-from AnyQt.QtCore import QTimer, QSize, Qt
-from AnyQt.QtWidgets import QWidget, QListWidget, QFileDialog
+from AnyQt.QtCore import QTimer, QSize, Qt, QItemSelection, QItemSelectionRange
+from AnyQt.QtGui import QBrush, QColor
+from AnyQt.QtWidgets import QListWidget, QFileDialog
 
 import Orange
 from Orange.util import scale
-from Orange.widgets import gui, widget, settings
+from Orange.widgets import gui, widget
+from Orange.widgets.settings import Setting, ContextSetting, DomainContextHandler
 from Orange.widgets.utils.colorpalette import ColorPaletteGenerator, GradientPaletteGenerator
+from Orange.widgets.widget import Input, Output
+from Orange.widgets.utils.itemmodels import VariableListModel
 
 from Orange.data import Table, DiscreteVariable, StringVariable
+from Orange.widgets.visualize.owdistributions import ScatterPlotItem
+from Orange.widgets.visualize.owscatterplotgraph import PaletteItemSample, DiscretizedScale
+
 import orangecontrib.network as network
-from orangecontrib.network.widgets.graphview import GraphView, Node, FR_ITERATIONS
+from orangecontrib.network.widgets.graphview import GraphView, Node, FR_ITERATIONS, LegendItem
 
 
 def non_reentrant(func):
@@ -30,6 +37,8 @@ def non_reentrant(func):
 
 
 CONTINUOUS_PALETTE = GradientPaletteGenerator('#00ffff', '#550066')
+MIN_NODE_SIZE = 1
+MAX_NODE_SIZE = 10
 
 
 class SelectionMode:
@@ -45,39 +54,25 @@ class SelectionMode:
     = range(9)  # FML
 
 
-class Output:
-    SUBGRAPH = 'Selected sub-network'
-    UNSELECTED_SUBGRAPH = 'Remaining sub-network'
-    DISTANCE = 'Distance matrix'
-    SELECTED = 'Selected items'
-    HIGHLIGHTED = 'Highlighted items'
-    REMAINING = 'Remaining items'
-    all = (SUBGRAPH, DISTANCE, SELECTED, HIGHLIGHTED, REMAINING)
-
-
 class OWNxExplorer(widget.OWWidget):
     name = "Network Explorer"
     description = "Visually explore the network and its properties."
     icon = "icons/NetworkExplorer.svg"
     priority = 6420
 
-    inputs = [
-        ("Network", network.Graph, "set_graph", widget.Default),
-        ("Node Subset", Table, 'set_marking_items'),
-        ("Node Data", Table, "set_items"),
-        ("Node Distances", Orange.misc.DistMatrix, "set_items_distance_matrix"),
-    ]
+    class Inputs:
+        network = Input("Network", network.Graph, default=True)
+        node_subset = Input("Node Subset", Table)
+        node_data = Input("Node Data", Table)
+        node_distances = Input("Node Distances", Orange.misc.DistMatrix)
 
-    outputs = [(Output.SUBGRAPH, network.Graph),
-               (Output.UNSELECTED_SUBGRAPH, network.Graph),
-               (Output.DISTANCE, Orange.misc.DistMatrix),
-               (Output.SELECTED, Table),
-               (Output.HIGHLIGHTED, Table),
-               (Output.REMAINING, Table)]
-
-    settingsList = ["lastVertexSizeColumn", "lastColorColumn",
-                    "lastLabelColumns", "lastTooltipColumns",]
-    # TODO: set settings
+    class Outputs:
+        subgraph = Output("Selected sub-network", network.Graph)
+        unselected_subgraph = Output("Remaining sub-network", network.Graph)
+        distances = Output("Distance matrix", Orange.misc.DistMatrix)
+        selected = Output("Selected items", Table)
+        highlighted = Output("Highlighted items", Table)
+        remaining = Output("Remaining items", Table)
 
     UserAdviceMessages = [
         widget.Message('When selecting nodes on the Marking tab, '
@@ -95,19 +90,25 @@ class OWNxExplorer(widget.OWWidget):
                        widget.Message.Information),
     ]
 
-    do_auto_commit = settings.Setting(True)
-    maxNodeSize = settings.Setting(50)
-    minNodeSize = settings.Setting(8)
-    selectionMode = settings.Setting(SelectionMode.FROM_INPUT)
-    tabIndex = settings.Setting(0)
-    showEdgeWeights = settings.Setting(False)
-    relativeEdgeWidths = settings.Setting(False)
-    invertNodeSize = settings.Setting(False)
-    markDistance = settings.Setting(1)
-    markSearchString = settings.Setting("")
-    markNBest = settings.Setting(1)
-    markNConnections = settings.Setting(2)
+    settingsHandler = DomainContextHandler()
 
+    do_auto_commit = Setting(True)
+    selectionMode = Setting(SelectionMode.FROM_INPUT)
+    tabIndex = Setting(0)
+    showEdgeWeights = Setting(False)
+    relativeEdgeWidths = Setting(False)
+    invertNodeSize = Setting(False)
+    markDistance = Setting(1)
+    markSearchString = Setting("")
+    markNBest = Setting(1)
+    markNConnections = Setting(2)
+
+    point_width = Setting(10)
+    edge_width = Setting(1)
+    attr_size = ContextSetting(None)
+    attr_color = ContextSetting(None)
+    attrs_label = ContextSetting({})
+    attrs_tooltip = ContextSetting({})
     graph_name = 'view'
 
     class Warning(widget.OWWidget.Warning):
@@ -142,11 +143,6 @@ class OWNxExplorer(widget.OWWidget):
         self.verticesPerEdge = 0
         self.edgesPerVertex = 0
 
-        self.lastVertexSizeColumn = ''
-        self.lastColorColumn = ''
-        self.lastLabelColumns = set()
-        self.lastTooltipColumns = set()
-
         self.items_matrix = None
         self.number_of_nodes_label = 0
         self.number_of_edges_label = 0
@@ -173,34 +169,29 @@ class OWNxExplorer(widget.OWWidget):
 
         self.relayout_button = gui.button(box, self, 'Re-layout',
                                           callback=self.relayout, autoDefault=False)
-        self.view.positionsChanged.connect(lambda _: self.progressbar.advance())
+        self.view.positionsChanged.connect(lambda positions, progress:
+                                           self.progressbar.widget.progressBarSet(int(round(100 * progress))))
         def animationFinished():
             self.relayout_button.setEnabled(True)
             self.progressbar.finish()
         self.view.animationFinished.connect(animationFinished)
 
-        self.colorCombo = gui.comboBox(
-            box, self, "node_color_attr", label='Color:',
-            orientation='horizontal', callback=self.set_node_colors)
+        self.color_model = VariableListModel(placeholder="(Same color)")
+        self.color_combo = gui.comboBox(
+            box, self, "attr_color", label='Color:',
+            orientation='horizontal', callback=self.set_node_colors,
+            model=self.color_model)
 
-        self.invertNodeSizeCheck = self.maxNodeSizeSpin = QWidget()  # Forward declaration
-        self.nodeSizeCombo = gui.comboBox(
-            box, self, "node_size_attr",
-            label='Size:',
-            orientation='horizontal',
-            callback=self.set_node_sizes)
+        self.size_model = VariableListModel(placeholder="(Same size)")
+        self.size_combo = gui.comboBox(
+            box, self, "attr_size",
+            label='Size:', orientation='horizontal',
+            callback=self.set_node_sizes, model=self.size_model)
+        gui.hSlider(box, self, 'point_width', label="Symbol size:   ", minValue=1,
+                    maxValue=10, step=1, createLabel=False,
+                    callback=self.set_node_sizes)
         hb = gui.widgetBox(box, orientation="horizontal")
         hb.layout().addStretch(1)
-        self.minNodeSizeSpin = gui.spin(
-            hb, self, "minNodeSize", 1, 50, step=1, label="Min:",
-            callback=self.set_node_sizes)
-        self.minNodeSizeSpin.setValue(8)
-        gui.separator(hb)
-        self.maxNodeSizeSpin = gui.spin(
-            hb, self, "maxNodeSize", 10, 200, step=5, label="Max:",
-            callback=self.set_node_sizes)
-        self.maxNodeSizeSpin.setValue(50)
-        gui.separator(hb)
         self.invertNodeSizeCheck = gui.checkBox(
             hb, self, "invertNodeSize", "Invert",
             callback=self.set_node_sizes)
@@ -222,6 +213,9 @@ class OWNxExplorer(widget.OWWidget):
         self.checkbox_relative_edges = gui.checkBox(
             eb, self, 'relativeEdgeWidths', 'Relative edge widths',
             callback=self.set_edge_sizes)
+        gui.hSlider(eb, self, 'edge_width', label="Edge width: ", minValue=1,
+                    maxValue=10, step=1, createLabel=False,
+                    callback=self.set_edge_sizes)
         self.checkbox_show_weights = gui.checkBox(
             eb, self, 'showEdgeWeights', 'Show edge weights',
             callback=self.set_edge_labels)
@@ -272,9 +266,13 @@ class OWNxExplorer(widget.OWWidget):
         self.set_graph(None)
         self.set_selection_mode()
 
+    def sizeHint(self):
+        return QSize(800, 600)
+
     def commit(self):
         self.send_data()
 
+    @Inputs.node_distances
     def set_items_distance_matrix(self, matrix):
         assert matrix is None or isinstance(matrix, Orange.misc.DistMatrix)
         self.items_matrix = matrix
@@ -389,92 +387,61 @@ class OWNxExplorer(widget.OWWidget):
 
     def send_data(self):
         if not self.graph:
-            for output in Output.all:
-                self.send(output, None)
+            for output in dir(self.Outputs):
+                if not output.startswith('__'):
+                    getattr(self.Outputs, output).send(None)
             return
         selected = self.view.getSelected()
-        self.send(Output.SUBGRAPH,
-                  self.graph.subgraph(selected) if selected else None)
-        self.send(Output.UNSELECTED_SUBGRAPH,
+        self.Outputs.subgraph.send(self.graph.subgraph(selected) if selected else None)
+        self.Outputs.unselected_subgraph.send(
                   self.graph.subgraph(self.view.getUnselected()) if selected else self.graph)
-        self.send(Output.DISTANCE,
-                  self.items_matrix.submatrix(sorted(selected)) if self.items_matrix is not None and selected else None)
+        self.Outputs.distances.send(
+                  self.items_matrix.submatrix(sorted(selected))
+                  if self.items_matrix is not None and selected else None)
         items = self.graph.items()
         if not items:
-            self.send(Output.SELECTED, None)
-            self.send(Output.HIGHLIGHTED, None)
-            self.send(Output.REMAINING, None)
+            self.Outputs.selected.send(None)
+            self.Outputs.highlighted.send(None)
+            self.Outputs.remaining.send(None)
         else:
             highlighted = self.view.getHighlighted()
-            self.send(Output.SELECTED, items[sorted(selected), :] if selected else None)
-            self.send(Output.HIGHLIGHTED, items[sorted(highlighted), :] if highlighted else None)
+            self.Outputs.selected.send(items[sorted(selected), :] if selected else None)
+            self.Outputs.highlighted.send(items[sorted(highlighted), :] if highlighted else None)
             remaining = sorted(set(self.graph) - set(selected) - set(highlighted))
-            self.send(Output.REMAINING, items[remaining, :] if remaining else None)
+            self.Outputs.remaining.send(items[remaining, :] if remaining else None)
 
     def _set_combos(self):
         self._clear_combos()
         self.graph_attrs = self.graph.items_vars()
-        lastLabelColumns = self.lastLabelColumns
-        lastTooltipColumns = self.lastTooltipColumns
 
-        for var in self.graph_attrs:
-            if var.is_discrete or var.is_continuous:
-                self.colorCombo.addItem(gui.attributeIconDict[gui.vartype(var)], var.name, var)
+        self.color_model[:] = [None] + [v for v in self.graph_attrs if v.is_primitive()]
+        self.size_model[:] = [None] + [v for v in self.graph_attrs if v.is_continuous]
+        self.size_combo.setDisabled(not self.graph_attrs)
+        self.color_combo.setDisabled(not self.graph_attrs)
+        self.set_node_sizes()
+        self.set_node_colors()
+        self.set_edge_sizes()
 
-            if var.is_continuous:
-                self.nodeSizeCombo.addItem(gui.attributeIconDict[gui.vartype(var)], var.name, var)
-
-        self.nodeSizeCombo.setDisabled(not self.graph_attrs)
-        self.colorCombo.setDisabled(not self.graph_attrs)
-
-        for i in range(self.nodeSizeCombo.count()):
-            if self.lastVertexSizeColumn == \
-                    self.nodeSizeCombo.itemText(i):
-                self.node_size_attr = i
-                self.set_node_sizes()
-                break
-
-        for i in range(self.colorCombo.count()):
-            if self.lastColorColumn == self.colorCombo.itemText(i):
-                self.node_color_attr = i
-                self.set_node_colors()
-                break
-
-        if lastLabelColumns:
-            selection = QItemSelection()
-            model = self.attListBox.model()
-            for i in range(self.attListBox.count()):
-                if str(self.attListBox.item(i).text()) in lastLabelColumns:
-                    selection.append(QItemSelectionRange(model.index(i, 0)))
-            selmodel = self.attListBox.selectionModel()
-            selmodel.select(selection, selmodel.Select | selmodel.Clear)
-        else:
-            self.attListBox.selectionModel().clearSelection()
+        for columns, box in ((self.attrs_label, self.attListBox),
+                             (self.attrs_tooltip, self.tooltipListBox)):
+            columns = [var.name for var in columns]
+            if columns:
+                selection = QItemSelection()
+                model = box.model()
+                for i in range(box.count()):
+                    if str(box.item(i).text()) in columns:
+                        selection.append(QItemSelectionRange(model.index(i, 0)))
+                selmodel = box.selectionModel()
+                selmodel.select(selection, selmodel.Select | selmodel.Clear)
+            else:
+                box.selectionModel().clearSelection()
         self._on_node_label_attrs_changed()
-
-        if lastTooltipColumns:
-            selection = QItemSelection()
-            model = self.tooltipListBox.model()
-            for i in range(self.tooltipListBox.count()):
-                if self.tooltipListBox.item(i).text() in lastTooltipColumns:
-                    selection.append(QItemSelectionRange(model.index(i, 0)))
-            selmodel = self.tooltipListBox.selectionModel()
-            selmodel.select(selection, selmodel.Select | selmodel.Clear)
-        else:
-            self.tooltipListBox.selectionModel().clearSelection()
         self._clicked_tooltip_lstbox()
 
-        self.lastLabelColumns = lastLabelColumns
-        self.lastTooltipColumns = lastTooltipColumns
-
     def _clear_combos(self):
         self.graph_attrs = []
-
-        self.colorCombo.clear()
-        self.nodeSizeCombo.clear()
-
-        self.colorCombo.addItem('(none)', None)
-        self.nodeSizeCombo.addItem("(uniform)")
+        self.color_combo.clear()
+        self.size_combo.clear()
 
     def set_graph_none(self):
         self.graph = None
@@ -487,6 +454,7 @@ class OWNxExplorer(widget.OWWidget):
         self._items = None
         self.view.set_graph(None)
 
+    @Inputs.network
     def set_graph(self, graph):
         if not graph:
             return self.set_graph_none()
@@ -499,6 +467,7 @@ class OWNxExplorer(widget.OWWidget):
             self.Error.network_too_large()
             return
         self.information()
+        self.closeContext()
 
         all_edges_equal = bool(1 == len(set(w for u,v,w in graph.edges(data='weight'))))
         self.checkbox_show_weights.setEnabled(not all_edges_equal)
@@ -518,11 +487,14 @@ class OWNxExplorer(widget.OWWidget):
         self.edgesPerVertex = self.graph.number_of_edges() / max(1, self.graph.number_of_nodes())
 
         self._set_combos()
+        if self.graph.items():
+            self.openContext(self.graph.items().domain)
         self.Error.clear()
 
         self.set_selection_mode()
         self.relayout()
 
+    @Inputs.node_data
     def set_items(self, items=None):
         self._items = items
         if items is None:
@@ -538,12 +510,17 @@ class OWNxExplorer(widget.OWWidget):
         self.graph.set_items(items)
         self._set_combos()
 
+    @Inputs.node_subset
     def set_marking_items(self, items):
         self.markInputRadioButton.setEnabled(False)
         self.markInputItems = items
 
         self.Warning.clear()
 
+        if self.selectionMode == SelectionMode.FROM_INPUT and \
+                (items is None or self.graph is None or self.graph.items() is None):
+            self.selectionMode = SelectionMode.NONE
+
         if items is None:
             self.view.selectionChanged.emit()
             return
@@ -552,15 +529,7 @@ class OWNxExplorer(widget.OWWidget):
             self.Warning.no_graph_or_items()
             return
 
-        graph_items = self.graph.items()
-        domain = graph_items.domain
-
         if len(items) > 0:
-            commonVars = (set(x.name for x in chain(items.domain.variables,
-                                                    items.domain.metas))
-                          & set(x.name for x in chain(domain.variables,
-                                                      domain.metas)))
-
             self.markInputRadioButton.setEnabled(True)
         self.view.selectionChanged.emit()
 
@@ -580,7 +549,7 @@ class OWNxExplorer(widget.OWWidget):
 
     def _on_node_label_attrs_changed(self):
         if not self.graph: return
-        attributes = self.lastLabelColumns = [self.graph_attrs[i] for i in self.node_label_attrs]
+        attributes = self.attrs_label = [self.graph_attrs[i] for i in self.node_label_attrs]
         if attributes:
             table = self.graph.items()
             if not table: return
@@ -593,7 +562,7 @@ class OWNxExplorer(widget.OWWidget):
 
     def _clicked_tooltip_lstbox(self):
         if not self.graph: return
-        attributes = self.lastTooltipColumns = [self.graph_attrs[i] for i in self.tooltipAttributes]
+        attributes = self.attrs_tooltip = [self.graph_attrs[i] for i in self.tooltipAttributes]
         if attributes:
             table = self.graph.items()
             if not table: return
@@ -608,6 +577,8 @@ class OWNxExplorer(widget.OWWidget):
                 node.setTooltip(None)
 
     def set_edge_labels(self):
+        if not self.graph:
+            return
         if self.showEdgeWeights:
             weights = (str(w or '') for u, v, w in self.graph.edges(data='weight'))
         else:
@@ -617,9 +588,14 @@ class OWNxExplorer(widget.OWWidget):
 
     def set_node_colors(self):
         if not self.graph: return
-        self.lastColorColumn = self.colorCombo.currentText()
-        attribute = self.colorCombo.itemData(self.colorCombo.currentIndex())
+        attribute = self.attr_color
         assert not attribute or isinstance(attribute, Orange.data.Variable)
+        if self.view.legend is not None:
+            self.view.scene().removeItem(self.view.legend)
+            self.view.legend.clear()
+        else:
+            self.view.legend = LegendItem()
+            self.view.legend.set_parent(self.view)
         if not attribute:
             for node in self.view.nodes:
                 node.setColor(None)
@@ -637,17 +613,24 @@ class OWNxExplorer(widget.OWWidget):
         else: raise RuntimeError("Shouldn't be able to select this column")
         if attribute.is_continuous:
             colors = CONTINUOUS_PALETTE[scale(values)]
+            label = PaletteItemSample(CONTINUOUS_PALETTE,
+                                      DiscretizedScale(np.nanmin(values), np.nanmax(values)))
+            self.view.legend.addItem(label, "")
+            self.view.legend.setGeometry(label.boundingRect())
         elif attribute.is_discrete:
             DISCRETE_PALETTE = ColorPaletteGenerator(len(attribute.values))
             colors = DISCRETE_PALETTE[values]
+            for value, color in zip(attribute.values, DISCRETE_PALETTE):
+                self.view.legend.addItem(
+                    ScatterPlotItem(pen=Node.Pen.DEFAULT, brush=QBrush(QColor(color)), size=10,
+                                    symbol="o"), escape(value))
         for node, color in zip(self.view.nodes, colors):
             node.setColor(color)
+        self.view.scene().addItem(self.view.legend)
+        self.view.legend.geometry_changed()
 
     def set_node_sizes(self):
-        attribute = self.nodeSizeCombo.itemData(self.nodeSizeCombo.currentIndex())
-        depending_widgets = (self.invertNodeSizeCheck, self.maxNodeSizeSpin)
-        for w in depending_widgets:
-            w.setDisabled(not attribute)
+        self.invertNodeSizeCheck.setDisabled(not self.attr_size)
 
         if not self.graph:
             return
@@ -656,10 +639,11 @@ class OWNxExplorer(widget.OWWidget):
             return
 
         try:
-            values = table.get_column_view(attribute)[0]
+            a = table.get_column_view(self.attr_size)[0]
+            values = a.copy()
         except Exception:
             for node in self.view.nodes:
-                node.setSize(self.minNodeSize)
+                node.setSize(MIN_NODE_SIZE * self.point_width)
             return
 
         if self.invertNodeSize:
@@ -668,23 +652,23 @@ class OWNxExplorer(widget.OWWidget):
         nodemin, nodemax = np.nanmin(values), np.nanmax(values)
         if nodemin == nodemax:
             # np.polyfit borks on this condition
-            sizes = (self.minNodeSize for i in range(len(self.view.nodes)))
+            sizes = (MIN_NODE_SIZE for _ in range(len(self.view.nodes)))
         else:
             k, n = np.polyfit([nodemin, nodemax],
-                              [self.minNodeSize, self.maxNodeSize], 1)
+                              [MIN_NODE_SIZE, MAX_NODE_SIZE], 1)
             sizes = values * k + n
             sizes[np.isnan(sizes)] = np.nanmean(sizes)
         for node, size in zip(self.view.nodes, sizes):
-            node.setSize(size)
+            node.setSize(size * self.point_width)
 
     def set_edge_sizes(self):
         if not self.graph: return
         if self.relativeEdgeWidths:
             widths = [self.graph.adj[u][v].get('weight', 1)
                       for u, v in self.graph.edges()]
-            widths = scale(widths, .7, 8)
+            widths = scale(widths, .7, 8) * np.log2(self.edge_width/4 + 1)
         else:
-            widths = (.7 for i in range(self.graph.number_of_edges()))
+            widths = (.7 * self.edge_width for _ in range(self.graph.number_of_edges()))
         for edge, width in zip(self.view.edges, widths):
             edge.setSize(width)
 
@@ -707,26 +691,18 @@ class OWNxExplorer(widget.OWWidget):
 
 if __name__ == "__main__":
     from AnyQt.QtWidgets import QApplication
-    import sys
-    a = QApplication(sys.argv)
+    a = QApplication([])
     ow = OWNxExplorer()
     ow.show()
 
-    def setNetwork(signal, data, id=None):
-        if signal == 'Network':
-            ow.set_graph(data)
-        #if signal == 'Items':
-        #    ow.set_items(data)
+    def set_network(data, id=None):
+        ow.set_graph(data)
 
     import OWNxFile
     from os.path import join, dirname
     owFile = OWNxFile.OWNxFile()
-    owFile.send = setNetwork
+    owFile.Outputs.network.send = set_network
     owFile.openNetFile(join(dirname(dirname(__file__)), 'networks', 'leu_by_genesets.net'))
-    #~ owFile.openFile(join(dirname(dirname(__file__)), 'networks', 'airtraffic.net'))
-    #~ owFile.openFile(join(dirname(dirname(__file__)), 'networks', 'lastfm.net'))
-    #~ owFile.show()
-    #~ owFile.selectNetFile(0)
 
     a.exec_()
     ow.saveSettings()
